/**
 * Cécile Janin dg408 driver
 */

#define DT_DRV_COMPAT vishay_dg408

#include <app/drivers/dg408.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/logging/log.h>
#include <zephyr/init.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(dg408, CONFIG_DG408_LOG_LEVEL);

struct dg408_config {
	const struct gpio_dt_spec cmd_gpios[4];
};

static int dg408_init(const struct device *dev)
{
	int ret = 0;
	const struct dg408_config *config = dev->config;

	for (int i = 0; i < 3; ++i)
	{
		if (!gpio_is_ready_dt(&config->cmd_gpios[i]))
		{
			LOG_ERR("gpio not ready");
			return -ENODEV;
		}
		ret = gpio_pin_configure_dt(&config->cmd_gpios[i], GPIO_OUTPUT);
		ret = gpio_pin_set_dt(&config->cmd_gpios[i], 0);
	}
	return ret;
}

int dg408_set(const struct device *dev, uint8_t output_selected){
	
	const struct dg408_config *config = dev->config;
	int ret = 0;
	uint8_t val = 0;

	if ((dev == NULL) | (output_selected < 0) | (output_selected > 4))
	{
		LOG_ERR("Bad argument");
		return -EINVAL;
	}

	output_selected -= 1;

	val = output_selected;
	LOG_WRN("first val : %x", (val & 1));
	val = val >> 1;
	LOG_WRN("second val : %x", (val & 1));


	//ret = gpio_pin_set_dt(&config->cmd_gpios[output_selected], 1);


	if (ret)
	{
		LOG_ERR("enable error");
	}

    return ret;
}

#define DG408_DEFINE(node_id)                               \
	static const struct dg408_config config##node_id = {    \
		.cmd_gpios[0] = GPIO_DT_SPEC_GET_OR(node_id, a0_gpios, {0}),  \
		.cmd_gpios[1] = GPIO_DT_SPEC_GET_OR(node_id, a1_gpios, {0}),  \
	};                                                          \
                                                                \
	DEVICE_DT_DEFINE(node_id, dg408_init, NULL,NULL, &config##node_id,  \
			 POST_KERNEL, CONFIG_DG408_INIT_PRIORITY , NULL);

DT_FOREACH_STATUS_OKAY(vishay_dg408, DG408_DEFINE);
