/**
 * Cécile Janin dg441 driver
 */

#define DT_DRV_COMPAT vishay_dg441

#include <app/drivers/dg441.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/logging/log.h>
#include <zephyr/init.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(dg441, CONFIG_DG441_LOG_LEVEL);

struct dg441_config {
	const struct gpio_dt_spec cmd_gpios[4];
};

static int dg441_init(const struct device *dev)
{
	int ret = 0;
	const struct dg441_config *config = dev->config;

	for (int i = 0; i < 4; ++i)
	{
		if (!gpio_is_ready_dt(&config->cmd_gpios[i]))
		{
			LOG_ERR("gpio not ready");
			return -ENODEV;
		}
		ret = gpio_pin_configure_dt(&config->cmd_gpios[i], GPIO_OUTPUT);
		ret = gpio_pin_set_dt(&config->cmd_gpios[i], 0);
	}
	return ret;
}

int dg441_enable(const struct device *dev, uint8_t output_selected){
	const struct dg441_config *config = dev->config;

	int ret = 0;

	ret = gpio_pin_set_dt(&config->cmd_gpios[output_selected], 1);

	if (ret)
	{
		LOG_ERR("enable error");
	}

    return ret;
}

int dg441_enable_only(const struct device *dev, uint8_t output_selected){
	const struct dg441_config *config = dev->config;

	int ret = 0;

	for (int i = 0; i < 4; ++i)
	{
		ret = gpio_pin_set_dt(&config->cmd_gpios[output_selected], 0);
		if (ret)
		{
			LOG_ERR("loop disable error");
		}
	}

	ret = gpio_pin_set_dt(&config->cmd_gpios[output_selected], 1);

	if (ret)
	{
		LOG_ERR("enable error");
	}

    return ret;
}

int dg441_disable(const struct device *dev, uint8_t output_selected){
	const struct dg441_config *config = dev->config;

	int ret = 0;

	ret = gpio_pin_set_dt(&config->cmd_gpios[output_selected], 0);

	if (ret)
	{
		LOG_ERR("disable error");
	}

    return ret;
}

#define DG441_DEFINE(node_id)                               \
	static const struct dg441_config config##node_id = {    \
		.cmd_gpios[0] = GPIO_DT_SPEC_GET_OR(node_id, cmd_1_gpios, {0}),  \
		.cmd_gpios[1] = GPIO_DT_SPEC_GET_OR(node_id, cmd_2_gpios, {0}),  \
		.cmd_gpios[2] = GPIO_DT_SPEC_GET_OR(node_id, cmd_3_gpios, {0}),  \
		.cmd_gpios[3] = GPIO_DT_SPEC_GET_OR(node_id, cmd_4_gpios, {0}),  \
	};                                                          \
                                                                \
	DEVICE_DT_DEFINE(node_id, dg441_init, NULL,NULL, &config##node_id,  \
			 POST_KERNEL, CONFIG_DG441_INIT_PRIORITY , NULL);

DT_FOREACH_STATUS_OKAY(vishay_dg441, DG441_DEFINE);
