/**
 * C&écile Janin pca9546a driver
 */

#define DT_DRV_COMPAT nxp_pca9546a

#include <app/drivers/pca9546a.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/logging/log.h>
#include <zephyr/init.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(pca9546a, CONFIG_PCA9546A_LOG_LEVEL);

struct pca9546a_config {
	const struct i2c_dt_spec i2c;
	const struct gpio_dt_spec reset_gpios;
};

static int pca9546a_init(const struct device *dev)
{
	int ret = 0;
	const struct pca9546a_config *config = dev->config;

	if (!i2c_is_ready_dt(&config->i2c))
	{
		LOG_ERR("I2C bus not ready");
		return -ENODEV;
	}
	
	if (!gpio_is_ready_dt(&config->reset_gpios))
	{
		LOG_ERR("gpio not ready");
		return -ENODEV;
	}

	ret = gpio_pin_configure_dt(&config->reset_gpios, GPIO_OUTPUT);
	ret = gpio_pin_set_dt(&config->reset_gpios, 0);

	if (ret)
	{
		LOG_ERR("gpio conf error : %x", ret);
	}

	return ret;
}

int pca9546a_select(const struct device *dev, uint8_t output_selected){
	const struct pca9546a_config *config = dev->config;

	int ret = 0;

	ret = i2c_write_dt(&config->i2c, &output_selected, sizeof(output_selected));
	if (ret)
	{
		LOG_ERR("Failed to write configuration");
	}

    return ret;
}

#define PCA9546A_DEFINE(node_id)                               \
	static const struct pca9546a_config config##node_id = {    \
		.reset_gpios = GPIO_DT_SPEC_INST_GET_OR(node_id, reset_gpios, {0}),  \
		.i2c = I2C_DT_SPEC_INST_GET(node_id),	\
	};                                                          \
                                                                \
	DEVICE_DT_INST_DEFINE(node_id, pca9546a_init, NULL,NULL, &config##node_id,  \
			 POST_KERNEL, CONFIG_PCA9546A_INIT_PRIORITY , NULL);

DT_INST_FOREACH_STATUS_OKAY(PCA9546A_DEFINE);
