/**
 * Cecile Janin pca9574 driver
 */

/* MAL IMPLEMENTE, DEVRAIT ETRE AJOUTE EN TEMPS QUE GPIO */

#define DT_DRV_COMPAT nxp_pca9574

#include <app/drivers/pca9574.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/logging/log.h>
#include <zephyr/init.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(pca9574, CONFIG_PCA9574_LOG_LEVEL);

struct pca9574_config {
	const struct i2c_dt_spec i2c;
	const struct gpio_dt_spec int_gpios;
};

int pca9574_set_gpio_high(const struct device *dev, uint8_t gpio_nb)
{
	int ret = 0;
	const struct pca9574_config *config = dev->config;
	uint8_t buffer_i2c[2];

	if (dev == NULL || gpio_nb > 7)
	{
		return -EINVAL;
	}
	buffer_i2c[0] = 0;
	buffer_i2c[1] = 0;

	/* Set pointer to the OUT register */
	buffer_i2c[0] = REGISTER_OUT;
	ret = i2c_write_dt(&config->i2c, &buffer_i2c[0], 1);

	/* read the register */
	ret = i2c_read_dt(&config->i2c, &buffer_i2c[1], 1);

	buffer_i2c[1] = buffer_i2c[1] | (1 << gpio_nb); 
	LOG_WRN("reg val r 1 : 0x%x", buffer_i2c[1]);
	ret = i2c_write_dt(&config->i2c, buffer_i2c, sizeof(buffer_i2c));

	return ret;
}

int pca9574_set_gpio_low(const struct device *dev, uint8_t gpio_nb)
{
	int ret = 0;
	const struct pca9574_config *config = dev->config;
	uint8_t buffer_i2c[2];

	if (dev == NULL || gpio_nb > 7)
	{
		return -EINVAL;
	}

	/* Set pointer to the OUT register */
	buffer_i2c[0] = REGISTER_OUT;
	ret = i2c_write_dt(&config->i2c, &buffer_i2c[0], 1);

	/* read the register */
	ret = i2c_read_dt(&config->i2c, &buffer_i2c[1], 1);

	buffer_i2c[1] = buffer_i2c[1] & ~(1 << gpio_nb); 
	LOG_WRN("reg val r 0 : 0x%x", buffer_i2c[1]);
	ret = i2c_write_dt(&config->i2c, buffer_i2c, sizeof(buffer_i2c));

	return ret;
}

int pca9574_setup(const struct device *dev)
{
	int ret = 0;
	const struct pca9574_config *config = dev->config;
	uint8_t buffer_i2c[2];

	buffer_i2c[0] = REGISTER_CFG;
	buffer_i2c[1] = 0x00; // Set every GPIO in output cfg
	ret = i2c_write_dt(&config->i2c, buffer_i2c, 
						sizeof(buffer_i2c));

	buffer_i2c[0] = REGISTER_OUT;
	ret = i2c_write_dt(&config->i2c, buffer_i2c, sizeof(buffer_i2c));

	return ret;
}

static int pca9574_init(const struct device *dev)
{
	int ret = 0;
	const struct pca9574_config *config = dev->config;

	if (dev == NULL)
	{
		return -EINVAL;
	}

	if (!i2c_is_ready_dt(&config->i2c))
	{
		LOG_ERR("I2C bus not ready");
		return -ENODEV;
	}

	return ret;
}

#define PCA9574_DEFINE(node_id)                               \
	static const struct pca9574_config config##node_id = {    \
		.int_gpios = GPIO_DT_SPEC_INST_GET_OR(node_id, int_gpios, {0}),  \
		.i2c = I2C_DT_SPEC_INST_GET(node_id),	\
	};                                                          \
                                                                \
	DEVICE_DT_INST_DEFINE(node_id, pca9574_init, NULL,NULL, &config##node_id,  \
			 POST_KERNEL, CONFIG_PCA9574_INIT_PRIORITY , NULL);

DT_INST_FOREACH_STATUS_OKAY(PCA9574_DEFINE);
