#ifndef DRIVER_DG441_
#define DRIVER_DG441_

#include <zephyr/kernel.h>

#define DG441_OUTPUT_1 0
#define DG441_OUTPUT_2 1
#define DG441_OUTPUT_3 2
#define DG441_OUTPUT_4 3

/**
 * @brief Enable an output
 * 
 * Enable the given switch of the given device
 * 
 * @param dev Pointer to the device
 * @param output_selected the output number to enable
 * 
 * @retval 0 on success;
 * @retval <0 negative errno code on error. 
 */
int dg441_enable(const struct device *dev, 
        uint8_t output_selected);

/**
 * @brief Enable only an output
 * 
 * Enable the given switch of the given device, disbale others
 * 
 * @param dev Pointer to the device
 * @param output_selected the output number to enable
 * 
 * @retval 0 on success;
 * @retval <0 negative errno code on error. 
 */
int dg441_enable_only(const struct device *dev, 
        uint8_t output_selected);

/**
 * @brief Disable an output
 * 
 * Disable the given switch of the given device
 * 
 * @param dev Pointer to the device
 * @param output_selected the output number to enable
 * 
 * @retval 0 on success;
 * @retval <0 negative errno code on error. 
 */
int dg441_disable(const struct device *dev, 
        uint8_t output_selected);

#endif //DRIVER_PDG441_