#include <zephyr/shell/shell.h>
#include <stdlib.h>
#include <string.h>
#include <zephyr/sys/util.h>
#include <app/drivers/ad9953.h>

#define ARGV_DEV    1
#define ARGV_TYPE    2
#define ARGV_VAL    3

static int cmd_set(const struct shell *shell_ctx,
    size_t argc, char **argv)
{
    const struct device *dev;
    uint32_t val = 0;

    dev = device_get_binding(argv[ARGV_DEV]);

    if (!dev) {
        shell_error(shell_ctx, "Device driver %s not found.",
            argv[ARGV_DEV]);
        return -ENODEV;
    } else if (!device_is_ready(dev))
    {
        shell_error(shell_ctx, "Device driver %s not ready.",
            argv[ARGV_DEV]);
        return -ENODEV;
    }

    if (!strcmp(argv[ARGV_TYPE], "freq"))
    {
        if (argc != 4)
        {
            shell_error(shell_ctx, "Please give a value in Hz");
            return -EINVAL;
        }

        val = atoi(argv[ARGV_VAL]);
        ad9953_set_freq_tuning_word(dev, val);
        shell_print(shell_ctx, "Device %s frequency : %d", argv[ARGV_DEV], val);
    } else if (!strcmp(argv[ARGV_TYPE], "pll"))
    {
        if (argc != 4)
        {
            shell_error(shell_ctx, "Please give a value in between 0 and 20");
            return -EINVAL;
        }

        val = atoi(argv[ARGV_VAL]);
        ad9953_set_pll(dev, val);
    }
    
    return 0;
}

static int cmd_disable(const struct shell *shell_ctx,
    size_t argc, char **argv)
{
    const struct device *dev;

    dev = device_get_binding(argv[ARGV_DEV]);

    if (!dev) {
        shell_error(shell_ctx, "Device driver %s not found.",
            argv[ARGV_DEV]);
        return -ENODEV;
    } else if (!device_is_ready(dev))
    {
        shell_error(shell_ctx, "Device driver %s not ready.",
            argv[ARGV_DEV]);
        return -ENODEV;
    }

    ad9953_disable(dev);
    
    return 0;
}

SHELL_STATIC_SUBCMD_SET_CREATE(sub_set_cmds,
    SHELL_CMD(freq, NULL,
      "Set the output frequency of the generator, in Hz \nfreq [freq in Hz]",
      NULL),
    SHELL_CMD(pll, NULL,
      "Set the internal pll of the generator, in Hz \npll [freq in Hz]",
      NULL),
    SHELL_SUBCMD_SET_END  
);

static void device_name_get(size_t idx, struct shell_static_entry *entry)
{
    const struct device *dev = shell_device_lookup(idx, NULL);

    entry->syntax = (dev != NULL) ? dev->name : NULL;
    entry->handler = NULL;
    entry->help = "Select a device to set";
    entry->subcmd = &sub_set_cmds;
}

SHELL_DYNAMIC_CMD_CREATE(dsub_device, device_name_get);

SHELL_STATIC_SUBCMD_SET_CREATE(sub_freq_gen_cmds,
    SHELL_CMD_ARG(set, &dsub_device,
      "Set parameter of the generator : freq, pll",
      cmd_set, 3, 1),
    SHELL_CMD(disable, &dsub_device,
      "Disable the generator output",
      cmd_disable),
    SHELL_SUBCMD_SET_END  
);

SHELL_CMD_REGISTER(freqgen, &sub_freq_gen_cmds, "DDS AD9953 command set", NULL);
