#include <zephyr/kernel.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/logging/log.h>
#include <zephyr/usb/usb_device.h>
#include <zephyr/sys/ring_buffer.h>
#include <zephyr/drivers/led.h>
#include <app_version.h>

#include <app/drivers/pca9546a.h>
#include <app/drivers/dg441.h>
#include <app/drivers/ad9953.h>
#include <app/drivers/pca9574.h>
#include <app/drivers/dg408.h>

LOG_MODULE_REGISTER(main, CONFIG_APP_LOG_LEVEL);

#define LEDS_DEVICE DT_NODELABEL(leds)
#define PCA9546A_DEVICE DT_NODELABEL(pca9546a)
#define ABUS_DEVICE DT_NODELABEL(analog_bus)
#define AD9953_DEVICE DT_NODELABEL(ad9953)
#define PCA9574_DEVICE DT_NODELABEL(pca9574)
#define ADC_MUX_DEVICE DT_NODELABEL(adc_mux)

struct serial_peer
{
	const struct device *dev;
	struct serial_peer *data;
	struct ring_buf rb;
};

#define DEFINE_SERIAL_PEER(node_id) { \
		.dev = DEVICE_DT_GET(node_id),\
		},
static struct serial_peer peers[] = {
	DT_FOREACH_STATUS_OKAY(zephyr_cdc_acm_uart, DEFINE_SERIAL_PEER)};

BUILD_ASSERT(ARRAY_SIZE(peers) >= 2, "Not enough CDC ACM instances");

int main(void)
{
	int ret;
	const struct device *leds = DEVICE_DT_GET(LEDS_DEVICE);
	const struct device *pca9546a = DEVICE_DT_GET(PCA9546A_DEVICE);
	const struct device *abus = DEVICE_DT_GET(ABUS_DEVICE);
	const struct device *ad9953 = DEVICE_DT_GET(AD9953_DEVICE);
	const struct device *pca9574 = DEVICE_DT_GET(PCA9574_DEVICE);
	const struct device *adc_mux = DEVICE_DT_GET(ADC_MUX_DEVICE);
	

	LOG_INF("coucou de zephyr");

#if defined(CONFIG_USB_DEVICE_STACK_NEXT)
	if (enable_usb_device_next()!=0)
	{
		led_on(leds, 3);
	}
#else
	if (usb_enable(NULL) != 0)
	{
		led_on(leds, 3);
	}
#endif

	led_on(leds, 1);
	k_sleep(K_MSEC(2000));
	led_off(leds, 1);

	printk("Zephyr Example AD9953\n");

	if (pca9546a_select(pca9546a, OUTPUT_B1))
	{
		LOG_ERR("no config i2c mux");
	}

	if (dg441_enable_only(abus, DG441_OUTPUT_1))
	{
		LOG_ERR("no config abus");
	}

	pca9574_setup(pca9574);

	/* test AD 9953 */
	/* test AD 9953 */

	for (int i = 0; i < 4; ++i)
	{
		LOG_INF("output nb : %d", i);
		dg408_set(adc_mux, i);
	}

	while (1) {
		//pca9574_set_gpio_high(pca9574, 1);
		k_sleep(K_MSEC(1000));
		//pca9574_set_gpio_low(pca9574, 1);
		k_sleep(K_MSEC(1000));
	}

	return 0;
}

